/*
 * $Id: etimedate.cpp 1654 2006-06-09 13:44:34Z karijes $
 *
 * Etimedate, time-date configuration tool
 * Part of Equinox Desktop Environment (EDE).
 * Copyright (c) 2000-2006 EDE Authors.
 *
 * This program is licenced under terms of the 
 * GNU General Public Licence version 2 or newer.
 * See COPYING for details.
 */

#include "etimedate.h"
#include <efltk/Fl_Locale.h>
#include <edeconf.h>
static Fl_Window* timedateWindow;

static void cb_OK(Fl_Button*, void*) {
  saveTimeZone(); 
  timeBox->settime(); 
  exit(0);
}

static void cb_Apply(Fl_Button*, void*) {
  saveTimeZone(); 
  timeBox->settime();
}

static void cb_Cancel(Fl_Button*, void*) {
  exit(0);
}
Fl_Time* timeBox;
Fl_Choice* formatChoice;

#include <efltk/Fl_Image.h>

#include <efltk/Fl_Images.h>
static const char *datas_world[] = {
  "356 184 3 1",
  ". c #000078",
  "# c #007800",
  "a c #0000f8",
  "....................................................................................................................................................................................................................................................................................................................................................................",
  "....................................................................................................................................................................................................................................................................................................................................................................",
  "....................................................................................................................................................................................................................................................................................................................................................................",
  "....................................................................................................................................................................................................................................................................................................................................................................",
  "...................................................................................................................................................................................................................................................................................................................................................############.....",
  "................................................................................................................................................................................................................................................................................................................................................################....",
  ".........................................................................................................................................................................................................................................................................................................###############....................#######################.",
  ".......................................................................................................................................................................................................................................................................................................##################...............############################",
  "..................................................................................................................................................................................................................................................................................................########################.......###################################",
  "...####.....................................................................##...................................................................................................................................................................................................................########################...####################################.###",
  ".#######...................................................................###....##.............................#.............................................................................................................................................................................#########################..#########################################.",
  "########.........................................................#####....###..#.###............................####........................................................................................................................................................................##..####.##################..###########################################",
  "######................................###.###...................######...#..#.###.............................####.....................................................................................................................................................................#...####..####......##########..#############################################",
  "#####..........................#....#.#########...................###..##..##..#..............................###..##.................................................................................................................................................................###..######.....##############...#############################################",
  "####..........................########...####................................................................##..#####...........................................................................................................................................................####..##...#######..############.....##############################################",
  "##.............................########...........................................................................####...##.....................................................................................................................................................######.......######..##########.....################################################",
  "##.............................##.######..........................................................................###..#####...............................................................................................................................................##....######.##....#####..##########...##################################################",
  "##................................#####.##.............................................................................####.............................................................................................................................................#####.....##########.#####.#.##########..###################################################",
  "##...............................######.###...............................................................................#............................................................................................................................................######....#.###..#####.###..#.#########...###################################################",
  "###................................###...##..........................................##................................#####......................................................................................................................................####....##......#........#......##.#######........################################################",
  "###.................................#...............................................####...............................#######..................................................................................................................................#####..................#...##..#.###.#######.......#################################################",
  "#...............................................................................##.#####.......................##.#..#############..............................................................................................................................#####...##..####.########.#######.###..####..........######.########################################",
  "#.............................................................................######........................#########################......................#####...............................................................................................####....##########.#######....####...#.##..............##.....#######################################",
  "##...........................................................................######.......................###########################.....................#######...###...............................................................................................###########..######.##.#######.#####.....................#####################################",
  "#..........................................................................#####.........................##########################........................#####....###.................................................................................................######......###...###.######.####.......................####################################",
  "##.........................................................................####..........................########################...........................##.................................................................................................#######.................#...........##............................###################################",
  "#.........................................................................####.....................###.######################.#..##............................#..............................................................................................#########.........#...######..####.......##.###....................###################################",
  ".........................................................................####............#.........##########################...#########....###..............###.............................................................................................########.........###..#####..#####..##..########...................###################################",
  "........................................................................####.............##..###...##########################.#############.#######.............#.............................................................................................######..############..######..###..##############..................##################################.",
  "........................................................................###.............####.#####..#################################################.........###..#.........................................................................................######.##############..######......####..##########.................##################################.",
  "........................................................................####...........############..################################################........##########.............................#..........................................................###################....###..##....###...###########................#################################.",
  "............................................##.##.......................####..........#####.###############################################################.##############.........................####.......................###.##.............................###################......####...###.#.#############.................##############################.",
  "........................................##########.........................##.##.....#######.####################################################################################...................##.....................############..............................################.....#####..###.##.##############.............###############################..",
  ".....................................#############............................####...#######.######################################################################################.........#.............................##################...........##.##..........################..#######....##...###############............##############################...",
  "...................................###################.#.......................#####..######..######################################################################################.#####.########...................###########################..################...##########.####..###.#####.....#.......##########............##############################...",
  "...................................######################..............###..##.#####...#####..#######################################################################################################................##################################################.######..#.###..####.######...####.....########.................#########################....",
  ".................................##########################.....##...##################.####..############################################################################################################...........###################################################.##....###.#######.########..####....############...........########################.##.....",
  "..................................###########################..###..########################.##############################################################################################################..........###################################################################################....####.#########.........######################.#.........",
  ".................................############################..###.##########################################################################################################################################...........#################################################################################...###.##########.#.......#####################............",
  "...###...........................####################...####...##################################################################################################################################################....############################################.#.###################################........##############.......###################...........#.",
  "#######.........................######################......######################################################################################################################################################..#############################################..##################################..........#######..####........##################...........###",
  "########.......................##########..############.....######################################################################################################################################################..####################################################################################....############.#...........#############..#............###",
  "#######........................##########...###########..############################################################################################################################################...#####.##.....####################################################################################..##############.............###########.................##",
  "######.......................############...############..#########################################################################################################################################...................####################################################################################..##..#########.............###########..................#",
  ".##........................############...##########################################################################################################################################################..................##########################################################################....#...##........#####.#..............#########....................",
  ".............#............############...############################################################################################################################################################.................##########################################################################......##..#..........###................#######.....................",
  "............##...........#############..###########################################################################################################################################################..................#####################################################.###################.......###..#.######....#.................#######.....................",
  ".........................############...##########.#################################################################################################################################..#########.......................#######################################################################...............#######......................######.....................",
  ".........................#############..######..##..###########################################################################################################################..##a#########.........................#######################################################################...............#########....#.................####.....................",
  ".........................##############..####..##############################################################################################################################.....#####..##............................################...##################################################................#########....#...................#......................",
  ".................#........############....#...#########################################################################################################################.#####....#####....................................########.##...........############################..##############...............###########.####.........................................",
  ".............#...#........##...#######......#######################################################################################################################.##...###....#####.....................................########...............#############################################.............#################........................................",
  ".............#.##...........##.######....#.#######################################################################################################################.............#####.........................................####.#...............############################################..............################........................................",
  "..............###..........###.######...#######################################################################################################################.#.............#######.......................................####..#...............#################################################..........################.......................................",
  "..............###..........####.####....#####################################################################################################################.................#######....................................#####...##.................#################################################....##.#################.......................................",
  "...............###..........##..........####################################################################################################################.................#######....................................######........................##############################################.......##################.......................................",
  "............###.###.........###.##..######################################################################################################################...................######....##..............................###.#...........................#################################################...#####################....................................",
  "...........####..###........#############################################################################################################################....................######......#...........................#.##.............................#.################################################..#######################...................................",
  "..........####..#####.....#################################################################################################################################..#..#............#####..................................##................................##################################################...#######################..................................",
  "..........####..#####...#########################################################################################################################################.............###...............#.....................................................###################################################..#######################..................................",
  "..........####.######..#########################################################################################################################################..............##.......................................................................#..################################################.########################.................................",
  "...........##..######..##########################################################################################################################################.............#.............................................................................#############################################.#######################...................................",
  ".....................#############################################################################################################################################............#............................................................................####################################################################.....................................",
  "..................################################################################################################################################################..........................................................................................###########################################################.....##...##.................................",
  "...............##################################################################################################################################################..........#................................................................................###.##################################.##################.####.#....##..................................",
  "................################################################################################################################################################..........##..................................................................................###################################...################.#####.....####.................................",
  ".................###############################################################################################################################################.......####...................................................................................####################################...###################......#####.................................",
  "..................##########################################################################################################################################..........####....................................................................................#####################################...###############.#.......######................................",
  "...................################################.#######################################################################################################..........####......................................................................................#####################################...#.############........#######................................",
  "...................##############.################...##...#################################################################################################....##...###........................................................................................####################################.###..############............#.#................................",
  ".............####.##########..####.##############..........#############..#######.########################################################################.....###.............................................................................................####################################.###.###...######................................................",
  "............#############.#....####.############............###########..#######..#######################################################################......####............................................................................................####################################.################................................................",
  "............###########......#..####..#########.....###......##########.########..######################################################################......####.............................................................................................####################################.#####.##########................................................",
  "............###########.....##...####..#########..#####################.#############################################################################.........###.............................................................................................######################################################................................................",
  "............#########.......##.....###.########.#######################..###########################################################################..........#...............................................................................................###################################################...................................................",
  "............#########.##.....#.....##...###############################..#########################################################################...........###...............................................................................................#################################################....................................................",
  ".............#######.............#.##....####.##########################..###############################################################...#..####..........###...............................................................................................################################################.............................................##......",
  "..............#####.............##.......###..#########################...##############################################################.......####.........###.................................................................................................###############################################..............................................###....",
  "...............###...#####.####.###......##.#..########################....#################################################################....####.......####.................................................................................................##############################################................................................##....",
  "...................############............###..##...#.#####################################################################################....####....#######..................................................................................................##############################################.....................................................",
  "...............#################............##....#.##.###################################################################################.......##..#########...................................................................................................##############################################.....................................................",
  ".............##################.............##...####..####################################################################################.....###..#######......................................................................................................###########################################.......................................................",
  "............###################.........##........##..#####################################################################################.........#..####.........................................................................................................########################################........................................................",
  "...........########################.....###..#........######################################################################################.......#######...........................................................................................................#####################################..........................................................",
  "...........##########################..########.############################################################################################.......###.......##.......................................................................................................####################################..........................................................",
  "...........#################################################################################################################################........#.........##......................................................................................................###.###############################...........................................................",
  "...........###########################################.#####################################################################################..................#........................................................................................................######################.###...######..........................................................",
  "..........############################################.#####################################################################################...................#........................................##..............................................................##.#################...#.......###..........................................................",
  ".........#############################################.##############.######################################################################.......#...........#.........................................###............................................................###.###############............###..#.......................................................",
  ".......###############################################..##############.###...##############################################################......#.............##........................................###..............................................................##.#############..............###.##......................................................",
  "......#################################################.###############....#..############################################################......###.............##........................................................................................................###.############..............##...##.....................................................",
  "......#################################################..##############...##...#.###########################..############################.##....#.............###...............................................####......................................................##.############..............##...###....................................................",
  ".....##################################################...#############..####.........#####################....##########################.###...................#..................................................###......................................................##.##########....................###....................................................",
  ".....###################################################..####################.........####################....#########################..##........................................................................####........................................................#########..............####....#....................................................",
  "....#####################################################..####################.........###################....#######################....#............................................................................##.###.#...................................................########............######........................................................",
  "....######################################################.###################..........###################...#################.###.......#.............................................................................####..###.................................................########.......###...########.....................................................",
  "....#######################################################.##################...........#.##############.....################.##..............................................................................................##.................................................#########.....####........##...###................................................",
  ".....######################################################.#################..............#############.......###################..............................................................................................##.................................................#########...####..............####...............................................",
  ".....#######################################################.###############...............#############........#############..##.........#..........................................................................................................................................##############........####.######.##...........................................",
  ".....######################################################...##############...............###########...........############............###.......................#...................................................................................................................############..........#.........#..##........................................",
  ".....#######################################################..###########...................#########............#############...........###.......................#.....................................................................................................................########...##....................#.#.....................................#.",
  ".....########################################################.########......................#######..............#..###########...........##..............................................................................................................................................#.###########.....................##...................................##.",
  ".....########################################################..####..........................######............##...###########............##................................................................................................................................................##########......................#....................................#.",
  ".....##########################################################.##...........................#######..........###...############............#......................#...........................................................................................................................########......................#......................................",
  ".....###########################################################.............................#######...........##...###..#######...........#.#.....................#...............................................................................................................................###............###........#......................................",
  "......#############################################################.####......................######..........##....###..######..........#..#.##....................................................................................................................................................###.........#######......#......................................",
  "......#################################################################.......................####.............#...####...###...........##.#.#...........#..........................................................................................................................................###....#..###############.......................................",
  ".......################################################################........................##..#................###....##...........#...............##............................................................................................................................................#########################.....................................",
  ".......###############################################################............................###...............###................#....####.......###............................................................................................................................................#####.###################.....................................",
  "........##############################################################............................####...............###.....................####......##........................#..........................................................................................................................###################.....................................",
  ".........##############.#############################################.............................###.................##..............##......###......#........................###.........................................................................................................................####################....................................",
  "............########.....###########################################...............................#.............###..###...........#####........................................##.........................................................................................................................##########################..............................",
  "...........................########################################...............................................#####.##.........######........................................##.........................................................................................................................###########################.............................",
  "............................#######################################................................................####..##.......######..........................................#.........................................................................................................................###########################.............................",
  ".............................####################################...................................................###..###.....########......................................................................................................................................................#...........############################.............................",
  "..............................###################################....................................................###..##....##########..###............................................................##..................................................................................##..........############################.............................",
  ".............................##################################......................................................####.......#########.####.......##....................................................##.................................................................................###.........#############################.............................",
  "............................##################################........................................................######....#######..####.......####...................................................##..................................................................................##........###################################........................",
  ".............................################################..........................................................######...#######..####...##...####.###..................................................................................................................................##........####################################.......................",
  "..............................###############################.............##...........................................######....######.#####.....##..########............................................................................................................................................#####################################.....................",
  "...............................#############################.............###............................................#####......####..###...##.###.##########........##...............................................................................................................................##########################################.................",
  "................................###########################.............###...............................................###............###.......#....############...####..........................##........###.......................................................................................##########################################.................",
  ".................................##########################.............##.................................................####........................###.##########.####.##........................##........###........................................................................................############################################..............",
  "................................###########################.................................................................######..................#..##...#########.......#.........................##......####.........................................................................................###########################################..............",
  ".................................##########################..................................................................########.#....###.##...##......#########........####......................#.......##..........................................................................................###########################################..............",
  ".................................###########################...................................................................######.#######..#..............###..###........####.........................................................................................................................##########################################...............",
  ".................................############################..##.....................................................................#..###.###....................###..........###........................................................................................................................#########################################...............",
  "..................................###########################..##................................................................................................................###.........................................................................................................................#######################################................",
  "..................................###########################.......#................................................................................###........#...............................................###..........................................................................................######################################.................",
  ".................................############################.......###.............................................................................######.....###.....................................##......####...........................................................................................#####################################.................",
  ".................................############################......####............................................................................#######.....###......................................#.......##............................................................................................#####################################.................",
  "................................#############################....#####.........................................................................############....####.....................#......................................................................................................................####################################.................",
  "................................#############################...######........................................................................###############..#####...................###......................................................................................................................##################################..................",
  "................................############################....######........................................................................###############.######.....................#.........#.#............................................................................................................################################..................",
  ".................................##########################....#######......................................................................########################..............................###...............................................................................................................###############################.................",
  ".................................########################......######........#.............................................................#########################..............................####..............................................................................................................###############################.................",
  "..................................######################........#####.....##.#...........................................................#############################............................................................................#.................................................................##############################..................",
  "..................................#####################........######.....##...........................................................################################..............##..........................................................####...............................................................#############################...................",
  "..................................######################.......#####..................................................................##################################..............##..........................................................####..............................................................#############################...................",
  "...................................#####################.......#####.................................................................###################################..............###...........................................................#...............................................................###########################.....................",
  "...................................#####################.......#####................................................................#####################################..............................................................................##...........................................................#########################.......................",
  "...................................####################..........##.................................................................######################################............................................................................####.........................................................##########################.......................",
  "...................................###################..............................................................................#######################################.............................................................................##.........................................................#######################..........................",
  "...................................##################..............................................................................########################################........................................................................................................................................#######################..........................",
  "...................................##################...............................................................................########################################.......................................................................................................................................#######################..........................",
  "....................................#################...............................................................................########################################.......................................................................................................................................######################...........................",
  "....................................################.................................................................................#######################################.......................................................................................................................................#####################............................",
  "....................................###############...................................................................................#####################################........................................................................................................................................#####################............................",
  ".....................................##############...................................................................................#####################################........................................................................................................................................####################.............................",
  "......................................#############...................................................................................############....####################.........................................................................................................................................##################...............................",
  "......................................###########.....................................................................................############......###.##############.........................................................................................................................................##################...............................",
  "......................................#########.......................................................................................#######............#..#############..........................................................................................................................................###############.#................................",
  "........................................####.........................................................................................####..................##############.....................#....................................................................................................................##############...................................",
  "......................................................................................................................................##..................###############.....................##..................................................................................................................###############...................................",
  ".............................................................................................................................................................###########......................###.................................................................................................................###############...................................",
  "..............................................................................................................................................................##########.......................###...............................................................................................................###############....................................",
  "...............................................................................................................................................................#..###.#.........................####.............................................................................................................##############.....................................",
  "...................................................................................................................................................................##..........................#####.............................................................................................................############.......................................",
  "...............................................................................................................................................................................................####..............................................................................................................###########........................................",
  "...................................................................................................................................................................#............................##..............................................................................................................###########.........................................",
  "..................................................................................................................................................................###........................###.#................................................................................................................########..........................................",
  "...................................................................................................................................................................####......................###..................................................................................................................#######...........................................",
  "....................................................................................................................................................................##.....................####...................................................................................................................#######...........................................",
  "..........................................................................................................................................................................................#####..................................................................................................................########...........................................",
  ".........................................................................................................................................................................................#####..................................................................................................................########............................................",
  "........................................................................................................................................................................................#####...................................................................................................................#######.............................................",
  "........................................................................................................................................................................................####....................................................................................................................#########...........................................",
  "........................................................................................................................................................................................###.....................................................................................................................#########...........................................",
  "................................................................................................................................................................................................................................................................................................................#########...........................................",
  ".......................................................................................##.......................................................................................................................................................................................................................########............................................",
  "........................................................................................#.......................................................................................................................................................................................................................#######.............................................",
  "................................................................................................................................................................................................................................................................................................................######.......####...................................",
  ".................................................................................................................................................................................................................................................................................................................#####......#####...................................",
  "..........................................................................................#......................................................................................................................................................................................................................######......#......................................",
  ".........................................................................................###......................................................................................................................................................................................................................######............................................",
  "..........................................................................................##.......................................................................................................................................................................................................................######...........................................",
  "...................................................................................................................................................................................................................................................................................................................#######..........................................",
  ".....................................................................................................................................................................................................................................................................................................................####...........................................",
  "......................................................................................................................................................................................................................................................................................................................##............................................",
  "....................................................................................................................................................................................................................................................................................................................................................................",
  "....................................................................................................................................................................................................................................................................................................................................................................",
  "....................................................................................................................................................................................................................................................................................................................................................................",
  "....................................................................................................................................................................................................................................................................................................................................................................",
  "...................................................................................................................................................................................................................................................................................................................................................................."};
Fl_Input_Browser* timeZonesList;

int main (int argc, char **argv) {

  Fl_Window* w;
  fl_init_locale_support("etimedate", PREFIX"/share/locale");
   {Fl_Window* o = timedateWindow = new Fl_Window(410, 299, _("Time and date"));
    w = o;
    o->shortcut(0xff1b);
     {Fl_Group* o = new Fl_Group(0, 245, 413, 54);
       {Fl_Button* o = new Fl_Button(154, 29, 80, 25, _("&OK"));
        o->callback((Fl_Callback*)cb_OK);
      }
       {Fl_Button* o = new Fl_Button(241, 29, 80, 25, _("&Apply"));
        o->callback((Fl_Callback*)cb_Apply);
        o->tooltip(_("Set system time. ->Just root user!<-"));
      }
       {Fl_Button* o = new Fl_Button(328, 29, 80, 25, _("&Cancel"));
        o->callback((Fl_Callback*)cb_Cancel);
      }
      o->end();
    }
     {Fl_Tabs* o = new Fl_Tabs(3, 5, 405, 265);
      o->color((Fl_Color)0xfffffffe);
       {Fl_Group* o = new Fl_Group(1, 24, 403, 240, _("Time/date"));
         {Fl_Group* o = new Fl_Group(5, 5, 220, 200);
          o->box(FL_DOWN_BOX);
          o->color((Fl_Color)7);
           {Fl_Calendar* o = new Fl_Calendar(10, 5, 200, 190);
            o->text_font(fl_fonts+9);
            o->color((Fl_Color)0xffffff00);
            o->text_color((Fl_Color)18);
            o->label_size(10);
            o->text_size(14);
          }
          o->end();
        }
        new Fl_Clock(235, 5, 165, 165);
         {Fl_Time* o = timeBox = new Fl_Time(235, 180, 165, 25);
        }
         {Fl_Choice* o = formatChoice = new Fl_Choice(281, 211, 119, 25, _("Time Format:")); o->begin();
          o->add("12 hour", 0, set_format12);
          o->add("24 hour", 0, set_format24);
          o->value(get_format());
          o->end();
        }
        o->end();
      }
       {Fl_Group* o = new Fl_Group(1, 24, 403, 240, _("Timezones"));
        o->hide();
         {Fl_Group* o = new Fl_Group(10, 5, 385, 170);
          o->box(FL_DOWN_BOX);
          o->color((Fl_Color)0x7b00);
           {Fl_Box* o = new Fl_Box(10, 5, 350, 160);
            o->image(Fl_Image::read_xpm("icons/world.xpm", (const char**)datas_world));
            o->box(FL_FLAT_BOX);
            o->color((Fl_Color)0x8000);
          }
          o->end();
        }
         {Fl_Input_Browser* o = timeZonesList = new Fl_Input_Browser(12, 185, 383, 25); o->begin();
          o->type(1); fillTimeZones();
          getCurrentTimeZone();
          o->end();
        }
        o->end();
      }
      o->end();
    }
    o->end();
    o->resizable(o);
  }
  w->show(argc, argv);
  return  Fl::run();
}
